/**
* This sample demonstrate how to use the counter of the board
**/

#include "stdafx.h"

/* entry point */
int _tmain( int /*argc*/, _TCHAR* /*argv[]*/ )
{
	int doContinue = 1;
	INT returnValue = 0;
	HANDLE boardHandle = NULL;
	ULONG readValue = 0;

	/* Counter parameters */
	BYTE counterIndex = 0;
	BYTE enableInterrupt = 0;
	BYTE outputMode = 0;
	BYTE direction = 0;
	BYTE level = 0;
	WORD reloadValue = 0;

	/* open a board */
	returnValue = OpenBoard(&boardHandle);
	if (returnValue != 0)
	{
		printf("Error %d opening the board \n", returnValue);
		_getch();
		return -1;
	}

	/* ask the counter */
	ReadUlong("Which counter do you want to use? (0: counter 0, 1: counter 1, 2: counter 2 ) ", 0, 2, &readValue);
	switch ( readValue )
	{
		case 0:
			counterIndex = PCI3016_TIMER_0;
			break;
		case 1:
			counterIndex = PCI3016_TIMER_1;
			break;
		case 2:
			counterIndex = PCI3016_TIMER_2;
			break;
	}

	/* ask for output mode */
	ReadUlong("Which output mode do you want to use? (0: Output disabled, 1: Output enabled, 2: Inverted output) ", 0, 2, &readValue);
	switch ( readValue )
	{
		case 0:
			outputMode = PCI3016_OUTPUT_DISABLED;
			break;			
		case 1:
			outputMode = PCI3016_OUTPUT_ENABLED;
			break;
		case 2:
			outputMode = PCI3016_OUTPUT_INVERTED;
			break;
	}

	/* ask for interrupts */
	ReadUlong("Do you want to enable interrupts? (0: No, 1: Yes) ", 0, 1, &readValue);
	enableInterrupt = (UCHAR) readValue;
		
	/* ask for counter direction */
	ReadUlong("Which direction do you want to count? (0: Up, 1: Down) ", 0, 1, &readValue);
	switch ( readValue )
	{
		case 0:
			direction = PCI3016_COUNTER_DIRECTION_UP;
			break;
		case 1:	
			direction = PCI3016_COUNTER_DIRECTION_DOWN;
			break;
	}
	
	/* ask for level mode */
	ReadUlong("What level do you want to count? (0: High, 1: Low, 2: Both) ", 0, 2, &readValue);
	switch ( readValue )
	{
		case 0:
			level = PCI3016_COUNTER_HIGH_LEVEL;
			break;
		case 1:		
			level = PCI3016_COUNTER_LOW_LEVEL;	
			break;			
		case 2:		
			level = PCI3016_COUNTER_BOTH_LEVEL;	
			break;
	}

	/* ask for reload value */
	ReadUlong("Which reload value do you want to use? (1 to 65535) ", 1, 65535, &readValue);
	reloadValue = (USHORT) readValue;

	/* set the interrupt routine (if choosed )*/
	if ( enableInterrupt != 0 )
	{
		returnValue = i_PCI3016_SetBoardIntRoutine( boardHandle, InterruptRoutine );
		if ( returnValue != 0 )
		{
			printf("Error %d in i_PCI3016_SetBoardIntRoutine \n", returnValue);
			i_PCI3016_CloseBoard(boardHandle);
			_getch();
			return -1;
		}	
	}

	/* start the counter mode*/
	returnValue = i_PCI3016_InitAndStartCounterTimerWatchdog( boardHandle,
		PCI3016_COUNTER_FUNCTION,	/* start as counter */
		counterIndex, 
		enableInterrupt, 
		reloadValue, 
		0,							/* not used for counter */
		outputMode, 
		0,							/* not used for counter */
		direction, 
		level );

	/* error handling */
	if ( returnValue != 0 )
	{
		printf("Error %d in i_PCI3016_StartCounter \n", returnValue);
		if ( enableInterrupt != 0 )
		{
			i_PCI3016_ResetBoardIntRoutine(boardHandle);
		}
		i_PCI3016_CloseBoard(boardHandle);
		_getch();
		return -1;
	}

	printf("Press the space key to trigger the counter.\n");
	printf("Press any other key to exit the sample.\n");
	printf("\n");
	while ( doContinue == 1 )
	{
		BYTE counterStatus = 0;
		BYTE softwareTriggerStatus = 0;
		WORD value = 0;
		BYTE clear = 0;

		/* get the value of the timer */
		returnValue = i_PCI3016_ReadCounterStatus( boardHandle, counterIndex, &counterStatus, &clear, &softwareTriggerStatus, &value);

		/* error handling */
		if ( returnValue != 0 )
		{
			printf("Error %d in i_PCI3016_StopTimer \n", returnValue);
			i_PCI3016_StopCounterTimerWatchdog( boardHandle, counterIndex );
			if ( enableInterrupt != 0 )
			{
				i_PCI3016_ResetBoardIntRoutine(boardHandle);
			}
			i_PCI3016_CloseBoard(boardHandle);
			_getch();
			return -1;
		}

		/* display current value */
		printf("The timer status is %x, value is %u, software trigger is %x, clear is %x \r", counterStatus, value, softwareTriggerStatus, clear);

		/* handle user input */
		if ( _kbhit() )
		{
			switch(_getch())
			{
				/* space bar key */
				case 32:
					/* trigger the counter (i.e. set its value back to zero) */
					returnValue = i_PCI3016_TriggerCounterTimerWatchdog(boardHandle, counterIndex);
					if ( returnValue != 0 )
					{
						printf("Error %d in i_PCI3016_TriggerTimer                                            \n", returnValue);
						doContinue = 0;
					}
					printf("Timer has been triggered!                                            \n");
					break;
				default:
					doContinue = 0;
					break;
			}
		}

		Sleep(100);
	}

	/* stop the counter */
	returnValue = i_PCI3016_StopCounterTimerWatchdog( boardHandle, counterIndex );
	if (returnValue != 0)
	{
		printf("Error %d in i_PCI3016_StopTimer \n", returnValue);
	}
	else
	{
		printf("i_PCI3016_StopTimer OK \n");
	}

	/* reset the interrupt */
	if ( enableInterrupt )
	{
		returnValue = i_PCI3016_ResetBoardIntRoutine( boardHandle );
		if (returnValue != 0)
		{
			printf("Error %d in i_PCI3016_ResetBoardIntRoutine \n", returnValue);
		}
		else
		{
			printf("i_PCI3016_ResetBoardIntRoutine OK \n");
		}
	}

	/* close the board */
	returnValue = i_PCI3016_CloseBoard(boardHandle);
	if (returnValue != 0)
	{
		printf("Error %d in i_PCI3016_CloseBoard \n", returnValue);
	}
	else
	{
		printf("i_PCI3016_CloseBoard OK \n");
	}

	/* exit */
	printf("\n Press any key to exit the sample ...");
	_getch();

	return 0;
}

