/**
* This sample demonstrate how to use the clear and trigger functionalities of the counter
**/

#include "stdafx.h"

/**
* Entry point
**/
int _tmain( int /*argc*/, _TCHAR* /*argv[]*/ )
{
	INT returnValue = 0;
	HANDLE	boardHandle = NULL;
	BYTE counterIndex = 0;
	ULONG readValue = 0;

	/* call the util method to open a board */
	returnValue = OpenBoard(&boardHandle);
	if (returnValue != 0)
	{
		printf("Error while opening the board : %d \n", (int) returnValue);
		_getch();
		return -1;
	}

	/* display some useful informations */
	printf("* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * \n");
	printf("* Counter with clear and trigger sample.\n");
	printf("* This sample uses a counter with direction DOWN, output disabled, \n");
	printf("* a detection only on rising edge, and a reload value of 100. \n");
	printf("* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * \n\n");

	/* ask  the counter index */
	ReadUlong("Enter the index of the counter that you want to use (0, 1 or 2): ",0,2, &readValue);
	counterIndex = (BYTE)readValue;

	/* start the selected component as counter */
	returnValue = i_PCI3016_InitAndStartCounterTimerWatchdog( boardHandle,
		PCI3016_COUNTER_FUNCTION,
		counterIndex,
		0,		/* interrupt disabled */
		100,	/* reload value */
		0,		/* not used for counter */
		0,		/* output disabled */
		0,		/* not used for counter */
		1,		/* direction down */
		0 );	/* counter level high */

	/* endless loop */
	printf("\nCounter is started. Press 't' to trigger the counter 'c' to clear it and 'ESC' to exit this sample.\n", counterIndex);
	for (;;)
	{
		BYTE status = 0;
		BYTE clearStatus = 0;
		BYTE triggerStatus = 0;
		WORD value = 0;

		/* get the counter status */
		returnValue = i_PCI3016_ReadCounterStatus( boardHandle,
			(BYTE)counterIndex,
			&status,
			&clearStatus,
			&triggerStatus,
			&value);
		if ( returnValue != 0)
		{
			printf("Error while reading counter's status : %d \n", returnValue);
			/* stop the counter and exit */
			i_PCI3016_StopCounterTimerWatchdog(boardHandle, (BYTE)counterIndex);
			_getch();
			return -1;
		}

		/* display the value */
		printf("Value : %d. Clear : %d. Trigger : %d \t\t", (int)value, (int) clearStatus, (int) triggerStatus );

		/* check if a key was hit */
		if ( _kbhit() )
		{
			switch ( _getch() )
			{
				/* t or T */
				case 116:
				case 84:
					/* trigger the counter */
					returnValue = i_PCI3016_TriggerCounterTimerWatchdog( boardHandle, (BYTE)counterIndex );
					if ( returnValue != 0 )
					{
						printf("Error while triggering counter : %d \n", returnValue);
						/* stop the counter and exit */
						i_PCI3016_StopCounterTimerWatchdog(boardHandle, (BYTE)counterIndex);
						_getch();
						return -1;
					}
					break;
				/* c or C */
				case 99:
				case 67:
					/* clear the counter */
					returnValue = i_PCI3016_ClearCounter( boardHandle, (BYTE)counterIndex );
					if ( returnValue != 0 )
					{
						printf("Error while clearing counter : %d \n", returnValue);
						/* stop the counter and exit */
						i_PCI3016_StopCounterTimerWatchdog(boardHandle, (BYTE)counterIndex);
						_getch();
						return -1;
					}
					break;
				/* esc */
				case 27:
					/* stop the timer */
					returnValue = i_PCI3016_StopCounterTimerWatchdog(boardHandle, (BYTE)counterIndex);
					if ( returnValue != 0 )
					{
						printf("Error while stopping the counter logic : %d \n", (int) returnValue);
						_getch();
						return -1;
					}
					/* success */
					return 0;
			}
		}
		Sleep(100);
		DisplayRunningStatus();
	}
}

