using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace ReadTTLInputs
{
    public partial class ReadTTLInputs : Form
    {
        /// <summary>
        /// Handle to the first board
        /// </summary>
        PCI3016_NET.PCI3016 board;

        /// <summary>
        /// Usefull variable used to access the input status picture
        /// </summary>
        PictureBox[] inputStatusBox;
        byte port = 0;

        /// <summary>
        /// Constructor of the form
        /// </summary>
        public ReadTTLInputs()
        {
            InitializeComponent();
            inputStatusBox = new PictureBox[8];
            inputStatusBox[0] = input0StatusBox;
            inputStatusBox[1] = input1StatusBox;
            inputStatusBox[2] = input2StatusBox;
            inputStatusBox[3] = input3StatusBox;
            inputStatusBox[4] = input4StatusBox;
            inputStatusBox[5] = input5StatusBox;
            inputStatusBox[6] = input6StatusBox;
            inputStatusBox[7] = input7StatusBox;
        }

        /// <summary>
        /// Call at the first load
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ReadTTLInputs_Load(object sender, EventArgs e)
        {
             // Open the first board of the system
            this.board = PCI3016_NET.PCI3016.OpenBoard(0);
            // Error handling
            if (this.board == null)
            {
                MessageBox.Show("Error OpenBoard returned null");
                this.Close();
                return;
            }
        }

        /// <summary>
        /// The read timer ticks
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void readTimer_Tick(object sender, EventArgs e)
        {
            int returnValue = 0;
            UInt16 inputValue = 0;
            // We read the status of the outputs
            if ((returnValue = this.board.ReadTTLInputs(port, ref inputValue)) != 0)
            {
                MessageBox.Show("Error " + returnValue.ToString() + " in ReadTTLInputs");
                this.Close();
                return;
            }

            // We change the image given the new status
            for (int i = 0; i < 8; i++)
            {
                if (((inputValue >> i) & 1) == 1)
                {
                    inputStatusBox[i].Image = global::ReadTTLInputs.Properties.Resources.led_on;
                }
                else
                {
                    inputStatusBox[i].Image = global::ReadTTLInputs.Properties.Resources.led_off;
                }
            }

            this.ResumeLayout(false);
            this.PerformLayout();
        }

        /// <summary>
        /// Occurs when the user closes the form
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ReadTTLInputs_FormClosing(object sender, FormClosingEventArgs e)
        {
            // Stop the timer
            this.readTimer.Stop();
        }

        private void radioButton1_CheckedChanged(object sender, EventArgs e)
        {
            // Stop the timer
            this.readTimer.Stop();
            port = 0;
            label1.Text = "8";
            label2.Text = "9";
            label3.Text = "10";
            label4.Text = "11";
            label5.Text = "15";
            label6.Text = "14";
            label7.Text = "13";
            label8.Text = "12";
            // Start the timer
            this.readTimer.Start();
        }

        private void radioButton2_CheckedChanged(object sender, EventArgs e)
        {
            // Stop the timer
            this.readTimer.Stop();
            int returnValue = 0;
            port = 2;
            returnValue = this.board.SetTTLPortConfiguration();
            if (returnValue != 0)
            {
                this.Close();
            }
            label1.Text = "16";
            label2.Text = "17";
            label3.Text = "18";
            label4.Text = "19";
            label5.Text = "23";
            label6.Text = "22";
            label7.Text = "21";
            label8.Text = "20";
            // Start the timer
            this.readTimer.Start();
        }
    }
}