using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace SetTTLOutputs
{
    public partial class SetTTLOutputs : Form
    {
        /// <summary>
        /// Board object
        /// </summary>
        private PCI3016_NET.PCI3016 board;

        /// <summary>
        /// Useful array to access all the checkboxes
        /// </summary>
        private CheckBox[] outputCheckBoxes;


        /// <summary>
        /// Constructor of the form
        /// </summary>
        public SetTTLOutputs()
        {
            InitializeComponent();

            // Construct the useful array
            this.outputCheckBoxes = new CheckBox[8];
            this.outputCheckBoxes[0] = this.output0CheckBox;
            this.outputCheckBoxes[1] = this.output1CheckBox;
            this.outputCheckBoxes[2] = this.output2CheckBox;
            this.outputCheckBoxes[3] = this.output3CheckBox;
            this.outputCheckBoxes[4] = this.output4CheckBox;
            this.outputCheckBoxes[5] = this.output5CheckBox;
            this.outputCheckBoxes[6] = this.output6CheckBox;
            this.outputCheckBoxes[7] = this.output7CheckBox;
        }

        /// <summary>
        /// Occurs the first time the form loads
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SetTTLOutputs_Load(object sender, EventArgs e)
        {
            int returnValue = 0;
            uint mask = 0xFF;

            // Construct the board object
            this.board = PCI3016_NET.PCI3016.OpenBoard(0);

            // Check the result
            if (this.board == null)
            {
                MessageBox.Show("Error while opening board 0");
                this.Close();
                return;
            }

            // Set the output memory to on
            if ((returnValue = board.SetDigitalOutputMemoryOn()) != 0)
            {
                MessageBox.Show("Error while setting output memory to on : " + returnValue);
                this.Close();
                return;
            }

            if ((returnValue = board.SetTTLOutputsOff((UInt16)mask)) != 0)
            {
                MessageBox.Show("Error while setting output off : " + returnValue);
                this.Close();
                return;
            }
        }

        /// <summary>
        /// Change the checkbox value
        /// </summary>
        /// <param name="index"></param>
        private void CheckBoxChanged(int index)
        {
            int returnValue = 0;

            if (this.outputCheckBoxes[index].Checked == true)
            {
                // Set to on
                if (board == null)
                    return;

                returnValue = board.SetTTLOutputsOn((UInt16)(1 << index));
                if (returnValue!= 0)
                {
                    MessageBox.Show("Error while setting TTL outputs to on : " + returnValue);
                }
            }
            else
            { 
                // Set to off
                if ((returnValue = board.SetTTLOutputsOff((UInt16)(1 << index))) != 0)
                {
                    MessageBox.Show("Error while setting TTL outputs to off : " + returnValue);
                }
            }
        }

        private void output0CheckBox_CheckedChanged(object sender, EventArgs e)
        {
            this.CheckBoxChanged(0);
        }

        private void output1CheckBox_CheckedChanged(object sender, EventArgs e)
        {
            this.CheckBoxChanged(1);
        }

        private void output2CheckBox_CheckedChanged(object sender, EventArgs e)
        {
            this.CheckBoxChanged(2);
        }

        private void output3CheckBox_CheckedChanged(object sender, EventArgs e)
        {
            this.CheckBoxChanged(3);
        }

        private void output4CheckBox_CheckedChanged(object sender, EventArgs e)
        {
            this.CheckBoxChanged(4);
        }

        private void output5CheckBox_CheckedChanged(object sender, EventArgs e)
        {
            this.CheckBoxChanged(5);
        }

        private void output6CheckBox_CheckedChanged(object sender, EventArgs e)
        {
            this.CheckBoxChanged(6);
        }

        private void output7CheckBox_CheckedChanged(object sender, EventArgs e)
        {
            this.CheckBoxChanged(7);
        }
    }
}